@@ -1,22 +1,4 @@
-########################################################################
-#
-#  OnlineJudge::Progra - Programming Tasks Grading System
-#
-#  Copyright (c) 2011-2012 Israel Leiva A.
-#
-#  Author: Israel Leiva A.	<ileiva AT csrg DOT cl>
-#
-#  This program is free software; you can redistribute it and/or modify
-#  it under the same terms as Perl itself.
-#
-#  This program is distributed in the hope that it will be useful,
-#  but WITHOUT ANY WARRANTY; without even the implied warranty of
-#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-#  GNU General Public License for more details.
-#
-#  http://www.gnu.org/licenses/
-#
-########################################################################
+#!/usr/bin/perl
 package OnlineJudge::Progra;
 
 use File::Spec::Functions qw(catfile catdir); 
@@ -26,23 +8,22 @@ use File::Copy;
 use warnings;
 use strict;
 
-our $VERSION = '0.01';
+our $VERSION = '0.02';
 
-$0 = 'Progra';
+$0 = 'progra';
 
-# forbidden words
-# these cannot be in user's source code
+# these words cannot be in user's source code
 my @BADWORDS = ();
 
 # output and error files (only if running in background mode)
-my $output_file = 'output.log';
-my $error_file 	= 'error.log';
-my $log_file 	= 'messages.log';
+my $output_file	= 'output.log';
+my $error_file	= 'error.log';
+my $log_file		= 'messages.log';
 
 # defines how source codes must be compiled according to its language
 my $COMPILERS = {
-	'c' => '/usr/bin/gcc _SOURCECODE_ -o _BINARY_ > /dev/null 2>&1',
-	'cpp' => '/usr/bin/g++ _SOURCECODE_ -o _BINARY_ > /dev/null 2>&1',
+	'c'		=> '/usr/bin/gcc _SOURCECODE_ -o _BINARY_ > /dev/null 2>&1',
+	'cpp'	=> '/usr/bin/g++ _SOURCECODE_ -o _BINARY_ > /dev/null 2>&1',
 };
 
 # defines how programs must be executed according to its language
@@ -57,14 +38,15 @@ sub new {
 	my $class 	= shift;
 	my $self 	= {};
 
-	$self->{'background'}    = 1;
-	$self->{'verbose'}       = undef;
-	$self->{'pid_file'}      = undef;
-	$self->{'get_sub'}       = undef;
-	$self->{'update_sub'}    = undef;
-	$self->{'time_interval'} = 60; # 1 minute
-	$self->{'home'}          = '/tmp';
-	$self->{'diff_options'}  = 'biwBE';
+	$self->{'background'}    	= 1;
+	$self->{'log'}    			= 1;
+	$self->{'verbose'}       	= undef;
+	$self->{'pid_file'}      	= undef;
+	$self->{'get_sub'}       	= undef;
+	$self->{'update_sub'}    	= undef;
+	$self->{'time_interval'} 	= 60; 		# in seconds
+	$self->{'home'}          	= '/tmp';
+	$self->{'diff_options'}  	= 'biwBE';
 
 	bless $self, $class;
 
@@ -99,7 +81,7 @@ sub set_timeinterval {
 	if ( $value ) { $self->{'time_interval'} = $value; }
 }
 
-# set a new compiler or change a previous one
+# set a new compiler or replace a previous one
 sub set_compiler {
 	my $self = shift;
 	my $lang = shift;
@@ -117,6 +99,14 @@ sub set_exec {
 	$EXEC->{ $lang } = $exec;
 }
 
+# set logging (true by default)
+sub set_logging {
+	my $self 	= shift;
+	my $value 	= shift or undef;
+	
+	if ( defined($value) ) { $self->{'log'} = $value; }
+}
+
 # set diff options
 sub diff_options {
 	my $self 	= shift;
@@ -132,13 +122,12 @@ sub verbose {
 	if ( $value ) { $self->{'verbose'} = 1; }
 }
 
-# e.g. system(), exec(), etc.
+# e.g. system, exec, etc.
 sub load_badwords {
 	my $self = shift;
 	my $file = shift;
 	
-	open my $F, '<', $file or $self->error("Can't open badwords file: $file");
-	# read file as a one big line
+	open my $F, '<', $file or $self->error("can't open badwords file: $file");
 	local $/;
 	my $content = <$F>;
 	close $F;
@@ -148,36 +137,36 @@ sub load_badwords {
 }
 
 sub run {
-	my $self = shift;
-	my %args = @_;
+	my $self	= shift;
+	my %args	= @_;
 	
-	my $get_sub 	= $args{'get_sub'};
+	my $get_sub		= $args{'get_sub'};
 	my $update_sub 	= $args{'update_sub'};
 
 	if ( !(defined($get_sub)) or !(defined($update_sub)) ) {
-		$self->error('Missing get and/or update subroutines!');
+		$self->error('missing get and/or update subroutines!');
 	}
 	
 	$self->{'get_sub'}    = $get_sub;
 	$self->{'update_sub'} = $update_sub;
 	
-	my $home       = $self->{'home'};
-	my $background = $self->{'background'};
+	my $home       	= $self->{'home'};
+	my $background	= $self->{'background'};
 	
 	my $output 	= catfile( $home, $output_file );
 	my $errors 	= catfile( $home, $error_file );
-	$log_file 	= catfile( $home, $log_file );
+	$log_file 		= catfile( $home, $log_file );
 	my $pid_file;
 	
 	if ( !$background ) {
 		$pid_file = catfile( $home, $$.'.pid' );
 		
-		open my $F, '>', $pid_file or $self->error('Could not create PID file');
+		open my $F, '>', $pid_file or $self->error('could not create PID file');
 		print {$F} $$;
 		close $F; 	
 
 		# everything is OK
-		print ":: Progra running with pid $$\n";
+		print ":: progra running with pid $$\n";
 		
 		$self->{'pid_file'} = $pid_file;
 		
@@ -188,15 +177,15 @@ sub run {
 		my $pid = fork();
 		
 		if ( $pid ) {
-			# to stop Progra delete PID file
+			# to stop pogra delete PID file
 			$pid_file = catfile( $home, $pid.'.pid' );
 	
-			open my $F, '>', $pid_file or $self->error('Could not create PID file');
+			open my $F, '>', $pid_file or $self->error('could not create PID file');
 			print {$F} $pid;
 			close $F; 	
 
 			# everything is OK
-			print ":: Progra running with pid $pid\n";
+			print ":: progra running with pid $pid\n";
 		} 
 		elsif ( $pid == 0 ) {
 			chdir $home;
@@ -207,7 +196,7 @@ sub run {
 			
 			$| = 1;
 	
-			# wait a second, just in case...
+			# let's just wait a sec
 			sleep 1;
 			$pid_file = catfile( $home, $$.'.pid' );
 			$self->{'pid_file'} = $pid_file;
@@ -222,18 +211,18 @@ sub judge {
 	my $self = shift;
 	
 	my $get_sub 		= $self->{'get_sub'};
-	my $update_sub 		= $self->{'update_sub'};
-	my $time_interval 	= $self->{'time_interval'};
+	my $update_sub 	= $self->{'update_sub'};
+	my $time_interval = $self->{'time_interval'};
 	my $pid_file 		= $self->{'pid_file'};
 	
 	my $date = $self->get_date();
-	print ":: Progra started - $date\n";
+	print ":: progra started - $date\n";
 	
 	# main loop
 	while (1) {
 		if ( !(-e $pid_file) ) {
 			$date = $self->get_date();
-			print ":: Progra terminated - $date\n\n";
+			print ":: progra terminated - $date\n\n";
 			last;
 		}
 		
@@ -248,7 +237,7 @@ sub judge {
 			}
 		}
 		
-		# do not stress the server :-)
+		# do not stress
 		sleep $time_interval;
 		undef @requests;
 	}
@@ -262,10 +251,10 @@ sub process_request {
 	$r->{'comment'} 		= 'PC';
 	$r->{'grade'} 			= 0;
 	$r->{'timemarked'} 		= time();
-	# in case sourcecode fails before being tested
+	# in case source code fails before being tested
 	$r->{'executiontime'}	= 0;
 
-	# processing the request is divided in three/four steps:
+	# processing a request is divided in three/four steps:
 	#  - check for badwords in source code.
 	#  - compile source code (if needed)
 	#  - test user's program.
@@ -284,15 +273,16 @@ sub process_request {
 	$processed->{'comment'}       = $r->{'comment'};
 
 	if ( $self->{'verbose'} ) {
-		print "\nRequest: $processed->{'rid'}\nGrade: $processed->{'grade'}\n";
-		print "Execution time: $processed->{'executiontime'}\nTime marked: ";
-		print "$processed->{'timemarked'}\nComment: $processed->{'comment'}\n";
+		print "\nrequest: $processed->{'rid'}\ngrade: $processed->{'grade'}\n";
+		print "execution time: $processed->{'executiontime'}\ntime marked: ";
+		print "$processed->{'timemarked'}\ncomment: $processed->{'comment'}\n";
 	}
 	
 	return $processed;
 }
 
 # this should be complemented with stronger security policies
+# see TODO in the POD for mode detail
 sub check {
 	my $self 	= shift;
 	my $r 		= shift;
@@ -308,26 +298,28 @@ sub check {
 			if ( $has_badword ) {
 				# request comment is updated
 				$r->{'comment'} = "BW: $bw";
-				$self->log($r);
+				if( $self->{'log'} != 0 ) {
+					$self->log($r);
+				}
 				return;
 			}
 		}
 	} else {
-		$self->warn($r->{'rid'}." - Error while reading $r->{'sourcecode'}");
+		$self->warn($r->{'rid'}." - error while reading $r->{'sourcecode'}");
 		$r->{'comment'} = 'IE ('.$r->{'rid'}.')';
 	}
 }
 
 # get the compile string associated with a given language
 sub get_compile_string {
-	my $self        = shift;
-	my $sourcecode 	= shift;
-	my $binary      = shift;
-	my $lang        = shift;
+	my $self			= shift;
+	my $sourcecode	= shift;
+	my $binary			= shift;
+	my $lang			= shift;
 		
-	my $compile_string = $COMPILERS->{ $lang };
-	$compile_string =~ s/_SOURCECODE_/$sourcecode/;
-	$compile_string =~ s/_BINARY_/$binary/;
+	my $compile_string	= $COMPILERS->{ $lang };
+	$compile_string		=~ s/_SOURCECODE_/$sourcecode/;
+	$compile_string		=~ s/_BINARY_/$binary/;
 	
 	return $compile_string;
 }
@@ -357,12 +349,12 @@ sub compile {
 
 # get the execution string associated with a given language
 sub get_exec_string {
-	my $self        = shift;
-	my $sourcecode  = shift;
-	my $lang        = shift;
+	my $self			= shift;
+	my $sourcecode	= shift;
+	my $lang			= shift;
 	
-	my $exec_string = $EXEC->{ $lang };
-	$exec_string =~ s/_FILE_/$sourcecode/;
+	my $exec_string	= $EXEC->{ $lang };
+	$exec_string		=~ s/_FILE_/$sourcecode/;
 	
 	return $exec_string;
 }
@@ -388,13 +380,13 @@ sub test {
 	for ( my $i = 0; $i < $r->{'testcases'}; $i++ ) {
 				
 		if ( !(-e $systeminput.$i) ) {
-			$self->warn($r->{'rid'}." - Error: system input file does not exist: $systeminput$i");
+			$self->warn($r->{'rid'}." - error: system input file does not exist: $systeminput$i");
 			$r->{'comment'} = 'IE ('.$r->{'rid'}.')';
 			return;
 		}		
 		
 		if ( !(-e $systemoutput.$i) ) {
-			$self->warn($r->{'rid'}." - Error: system output file does not exist: $systemoutput$i");
+			$self->warn($r->{'rid'}." - error: system output file does not exist: $systemoutput$i");
 			$r->{'comment'} = 'IE ('.$r->{'rid'}.')';
 			return;
 		}
@@ -435,7 +427,9 @@ sub test {
 				
 		if ( $@ =~ /time limit exceeded/ ) {
 			$r->{'comment'} = 'TL';
-			$self->log($r);	
+			if( $self->{'log'} != 0 ) {
+				$self->log($r);
+			}	
 			return;
 		}
 
@@ -460,10 +454,10 @@ sub test {
 
 # returns true if files are equal
 sub compare {
-	my $self         = shift;
-	my $systemoutput = shift;
-	my $useroutput   = shift;
-	my $options      = $self->{'diff_options'};
+	my $self			= shift;
+	my $systemoutput	= shift;
+	my $useroutput	= shift;
+	my $options		= $self->{'diff_options'};
 
 	# diff command will return a true value if there are any 
 	# differences between the files.  The -b argument ignores 
@@ -479,7 +473,7 @@ sub clean {
 	my $self 	= shift;
 	my $r 		= shift;	
 	
-	# removes output file created by user's program
+	# remove output file created by user's program
 	unlink $r->{'userpath'}.'output';
 	if( $r->{'compile'} and (-e $r->{'binary'}) ) { unlink $r->{'binary'}; }	 
 }
@@ -489,39 +483,38 @@ sub warn {
         my $error 	= shift;
         my ( $package, $filename, $line, $sub, $dayname );
 
-        my $date 	= $self->get_date();
-        my $errstr 	= ":: [$date] > ";
-        $errstr 	.= "\"$error\"\n";
+        my $date		= $self->get_date();
+        my $errstr		= ":: [$date] > ";
+        $errstr		.= "\"$error\"\n";
 
         # we want to know where the error came from
-        $filename = ( caller(1) )[1];
-        $line     = ( caller(1) )[2];
-        $sub      = ( caller(1) )[3];
+        $filename	= ( caller(1) )[1];
+        $line		= ( caller(1) )[2];
+        $sub		= ( caller(1) )[3];
 
-        $errstr .= " - Called by $sub() in $filename line $line\n";
+        $errstr .= " - called by $sub() in $filename line $line\n";
         $errstr .= "\n";
 	
         warn $errstr;
 }
 
-# remember it dies
 sub error {
         my $self 	= shift;
         my $error 	= shift;
         my ( $package, $filename, $line, $sub );
 
-        my $date 	= $self->get_date();
-        my $errstr 	= ":: [$date] > ";
-        $errstr 	.= "\"$error\"\n";
+        my $date		= $self->get_date();
+        my $errstr		= ":: [$date] > ";
+        $errstr		.= "\"$error\"\n";
     
-        print ":: Progra terminated - $date\n\n";
+        print ":: progra terminated - $date\n\n";
 
         # we want to know where the error came from
         $filename 	= ( caller(1) )[1];
         $line 		= ( caller(1) )[2];
         $sub 		= ( caller(1) )[3];
 
-        $errstr .= " - Called by $sub() in $filename line $line\n";
+        $errstr .= " - called by $sub() in $filename line $line\n";
         $errstr .= "\n";
 
         die $errstr;
@@ -538,16 +531,18 @@ sub log {
         my $msg  = ":: [$date] > ";
         $msg    .= $r->{'comment'}." on ".$r->{'rid'}."\n";
     
-        print ":: Logging - $date\n";
+        print ":: logging - $date\n";
     
-        open my $F, '>>', $log_file or error('Cannot open messages.log file!');
+        open my $F, '>>', $log_file or error('cannot open messages.log file!');
         print {$F} $msg;
         close $F;
 	
-	# save the source code that caused logging
+		# save the source code that caused logging
         if ( $date =~ /.*\s+(\d+):(\d+)/ ) { ($hour, $min) = ($1, $2); }
 	
-        if ( !(-d catdir($home, 'logged')) ) { mkdir catdir($home, 'logged'); } 
+        if ( !(-d catdir($home, 'logged')) ) { 
+			mkdir catdir($home, 'logged'); 
+		} 
 	
         my $filename = catfile( $home, catfile( 'logged', $r->{'rid'}.'_'.$hour.$min.'.'.$r->{'lang'} ) );
         copy( $r->{'sourcecode'}, $filename );
@@ -563,11 +558,12 @@ sub get_date {
         $mon++;
 
         my $dayname = $days->[$wday];
-        $hour = '00' if ($hour == 0);
-        $hour = '0'.$hour if ($hour < 10);
-        $min = '00' if ($min == 0);
-        $min = '0'.$min if ($min < 10);
-        $mon = '0'.$mon if ($mon < 10);
+        
+        $hour		= '00' if ($hour == 0);
+        $hour		= '0'.$hour if ($hour < 10);
+        $min		= '00' if ($min == 0);
+        $min		= '0'.$min if ($min < 10);
+        $mon		= '0'.$mon if ($mon < 10);
         my $date 	= "$dayname $year/$mon/$mday $hour:$min";
     
         return $date;
@@ -579,11 +575,11 @@ __END__
 
 =head1 NAME
 
-OnlineJudge::Progra - Programming Tasks Grading System
+OnlineJudge::Progra - (Just Another) Programming Tasks Grading System
  
 =head1 VERSION
 
-Version 0.01
+Version 0.02
 
 =head1 SYNOPSIS
 
@@ -605,8 +601,8 @@ Version 0.01
 
  my $judge = OnlineJudge::Progra->new();
  
- $judge->set_home('/home/progra/'); # Progra home directory
- $judge->set_timeinterval(30);		# time in seconds
+ $judge->set_home('/home/progra/'); # progra's home
+ $judge->set_timeinterval(30);		# in seconds
  
  $judge->run(
 	get_sub => sub { &get },
@@ -616,33 +612,38 @@ Version 0.01
 
 =head1 DESCRIPTION
 
-Progra is an Online Judge capable of compile and test programs
-written to solve a programming task. 
+Progra is an online judge capable of compile and test programs
+written to solve a programming task.
  
 =head1 METHODS
 
 =head2 set_home(/home/foo/bar)
 
-Sets the home directory for Progra files (logs, pid file, etc). If not specified
-is set to '/tmp'. When Progra starts, it creates a .pid file in its home directory. 
-You should remove the .pid file created in this directory in order to stop Progra.
+Sets the home directory for progra files (logs, pid file, etc). If not specified,
+is set to '/tmp'. Once progra starts, it creates a .pid file in its home directory. 
+You should remove the .pid file created in this directory in order to stop progra.
 
 =head2 set_timeinterval(n)
 
-Time interval (in seconds) it waits to check again for new requests to 
-be processed. Default to 60 seconds.
+The time interval defines the amount of time (in seconds) progra waits to check 
+again for new requests to process. Default to 60 seconds.
 
 =head2 set_background(true/false)
 
-Defines whether Progra runs on background or not. In case background is
-set, then output and errors will be redirected to specific files
-found in Progra home directory (output.log and error.log). True by default.
+Defines if progra runs on background or not. In case background is
+set, the output and errors will be redirected to the output.log and error.log 
+files found in progra's home directory. True by default.
+
+=head2 set_logging(true/false)
+
+Defines if progra log risky situations or not (badwords and time limits).  
+Anything different than zero is considered true. True by default.
 
 =head2 set_compiler('ext', '/path/to/compiler _SOURCECODE_ -options _BINARY_')
 
 Defines how source codes of a given language must be compiled. The B<_SOURCECODE_>
 and B<_BINARY_> strings must be present (yes, with both underscores) and they
-would be internally replaced by the corresponding filenames. Example: 
+will be internally replaced by the corresponding filenames. Example: 
 
  # how a .c file should be compiled
  $judge->set_compiler('c', '/usr/bin/gcc _SOURCECODE_ -o _BINARY_')
@@ -656,7 +657,7 @@ If you want to avoid the output generated by compilation errors, you should
 add '> /dev/null 2>&1' at the end of the string, otherwise it would be redireced
 to standard output.
 
-=head2 $judge->set_exec('ext', 'string to execute _FILE_')
+=head2 set_exec('ext', 'string to execute _FILE_')
 
 Defines how programs of a given language must be executed. The B<_FILE_> string
 must be present and it would be internally replaced by the corresponding
@@ -686,12 +687,11 @@ ignores case differences.
 
 Open the specified file and load the badwords into memory. They must be
 separated by commas. This is quite basic and should be enhanced with 
-stronger security policies. 
+stronger security policies.
 
 Example: system, exec, popen, etc. 
 
-Be aware that system and system() are different words!. This method is
-mandatory.
+Be aware that system and system() are different words!
 
 =head2 verbose(true/false)
 
@@ -733,7 +733,7 @@ of hash references:
  )
   
 
-You B<MUST> specify a I<rid>, it identifies the processed request uniquelly.
+You B<MUST> specify a I<rid>, it identifies the processed request uniquely.
 
 You B<MUST> specify a I<language>. It cannot compile and/or execute a 
 source code/program if it does not know its language. The format used is 
@@ -782,50 +782,57 @@ request was processed. The execution time field is the average execution
 time of all test cases. The comment field is a string that represents the
 state of the request. The possible values are:
 
- AC - Accepted: the program passed all test cases.
- WA - Wrong Answer: the program didn't pass all test cases.
- PC - Processing: the source code is in the queue waiting to be processed.
- TL - Time Limit Exceeded: the program ran out of time.
- CE - Compilation Error: there was an error while trying to compile the source code.
- BW - Bad Word found: a forbidden word was found in the source code.
- IE - Internal Error: something went wrong in Progra, check the error.log file.
+ AC. Accepted, the program passed all test cases.
+ WA. Wrong Answer, the program didn't pass all test cases.
+ PC. Processing, the source code is in the queue waiting to be processed.
+ TL. Time Limit Exceeded, the program ran out of time.
+ CE. Compilation Error, there was an error while trying to compile the source code.
+ BW: $word. The forbidden word $word was found in the source code.
+ IE. Internal Error, something went wrong in progra, check the error.log file.
 
-Note: When a bad/forbidden word is found in the source code or a time limit happens 
+Note: when a bad/forbidden word is found in the source code or a time limit happens 
 when executing it, a directory named "logged" will be created and the source code
 that caused the security breach will be saved with the format "requestid_hourmin.ext"
 
-Progra will return the result of a request inmediately after is 
-processed, so the update subroutine must receive only one
-request at a time. Progra will call this subroutine for each request 
-processed.
+Progra will return the result of a request inmediately after processing it, 
+so the update subroutine must receive only one request at a time. Progra will call 
+this subroutine for each request processed.
 
 
 =head1 BUGS
 
 No bugs found so far.
 
+=head1 TODO
+
+There are lots of things to do:
+
+=over
+
+=item Implement some kind of sandbox for safely test and compile.
+
+=item Try with distributed processing.
+
+=item Create working modules that use progra.
+
+=back
+
 =head1 AUTHOR
 
-Israel Leiva A. - <ileiva AT csrg DOT cl>
+ israel leiva - <ilv AT cpan DOT org>
 
 =head1 COPYRIGHT
 
-Copyright (C) 2011-2012 Israel Leiva A.
+ This program is free software: you can redistribute it and/or modify
+ it under the terms of the GNU General Public License as published by
+ the Free Software Foundation, either version 3 of the License, or
+ (at your option) any later version.
 
-This program is free software: you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation, either version 3 of the License, or
-(at your option) any later version.
+ This program is distributed in the hope that it will be useful,
+ but WITHOUT ANY WARRANTY; without even the implied warranty of
+ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+ GNU General Public License for more details.
 
-This program is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-GNU General Public License for more details.
-
-http://www.gnu.org/licenses/
-
-=head1 SEE ALSO
-
-L<Time::HiRes>, L<File::Spec::Functions>, L<Proc::Killall>, L<File::Copy>
+ http://www.gnu.org/licenses/
 
 =cut
